/*  The file blink_last.asm contains the last two functions of the project */
/*  Each function is a separate overlay.  The functions are placed in      */
/*  separate overlays by declaring them in  different segments.            */
/*  The function blink_next_to_last is declared in code segment pm1_code   */
/*  and function blink_last_stage is declared in code segment pm2_code.    */
/*  They will both live, according to LDF in memory segment ovl_code and   */
/*  run in memory segment pm_code.                                         */

#include <def21262.h>
#include "ovlay.h"

/*  The following code will be the third of 4 overlay functions.  */
/*  This function toggles LEDs 2, 4, 5, and 7.                    */
/*  The value of these LEDs 2, and 7 are only changed in          */
/*  this function.                                                */

.SECTION/PM             pm1_code;
blink_next_to_last:
.GLOBAL blink_next_to_last;
// Place AD15:0 into FLAG mode to display the status of the FLAGs register.
ustat1=dm(SYSCTL);nop;
bit set ustat1 PPFLGS;
dm(SYSCTL)=ustat1;

// Execute NOP loop to slow execution for visual observation
lcntr=PAUSELENGTH;
    do (pc,1) until lce;
    nop;

// Toggle FLAGS 2, 4, 5, 7
bit tgl flags 0x11440000;

// Place AD15:0 into Parallel Port mode to access external memory.
ustat1=dm(SYSCTL);nop;
bit clr ustat1 PPFLGS;
dm(SYSCTL)=ustat1;

rts;
nop;
nop;

blink_next_to_last.end:


/*  The following code will be the third of 4 overlay functions.  */
/*  This function toggles LEDs 3, 4, 5, and 6                     */
/*  The value of these LEDs 3, and 6 are only changed in          */
/*  this function.                                                */


.SECTION/PM             pm2_code;

blink_last_stage:
.GLOBAL blink_last_stage;
// Place AD15:0 into FLAG mode to display the status of the FLAGs register.
ustat1=dm(SYSCTL);nop;
bit set ustat1 PPFLGS;
dm(SYSCTL)=ustat1;

// Execute NOP loop to slow execution for visual observation
lcntr=PAUSELENGTH;
    do (pc,1) until lce;
    nop;

// Toggle LEDs 3, 4, 5, 6
bit tgl flags 0x05500000;

// Place AD15:0 into Parallel Port mode to access external memory.
ustat1=dm(SYSCTL);nop;
bit clr ustat1 PPFLGS;
dm(SYSCTL)=ustat1;


rts;    /*finished*/
nop;
nop;
nop;
blink_last_stage.end:

/*_______________________________________________________________________*/



