/*  The OVLY_MGR.ASM file is the overlay manager.  When a symbol  */
/*  residing in overlay is referenced, the ovelay manager loads   */
/*  the overlay code and begins execution.  (This overlay manager */
/*  does not check to see if the overlay is already in internal   */
/*  memory.)  A DMA transfer is performed to load in the memory   */
/*  overlay.                                                      */

#include <def21262.h>

.SECTION/DM             dm_data;

/*  The following constants are defined by the linker.  */
/*  These constants contain the word size, live locaton */
/*  and run location of the overlay functions.          */

.EXTERN _ov_word_size_run_1;
.EXTERN _ov_word_size_run_2;
.EXTERN _ov_word_size_run_3;
.EXTERN _ov_word_size_run_4;
.EXTERN _ov_word_size_live_1;
.EXTERN _ov_word_size_live_2;
.EXTERN _ov_word_size_live_3;
.EXTERN _ov_word_size_live_4;
.EXTERN _ov_startaddress_1;
.EXTERN _ov_startaddress_2;
.EXTERN _ov_startaddress_3;
.EXTERN _ov_startaddress_4;
.EXTERN _ov_runtimestartaddress_1;
.EXTERN _ov_runtimestartaddress_2;
.EXTERN _ov_runtimestartaddress_3;
.EXTERN _ov_runtimestartaddress_4;

/*  Placing the linker constants in a structure so the overlay  */
/*  manager can use the appropriate constant based on the    */
/*  overlay id.                          */
#define PHYS_WORD_SIZE(run_size,live_size) (48 / (live_size/run_size))
.import "OverlayStruct.h";

.struct OverlayConstantsList _OverlayConstants = {
_ov_startaddress_1, _ov_startaddress_2,_ov_startaddress_3, _ov_startaddress_4,
_ov_runtimestartaddress_1, _ov_runtimestartaddress_2,_ov_runtimestartaddress_3, _ov_runtimestartaddress_4,
_ov_word_size_run_1,_ov_word_size_run_2,_ov_word_size_run_3,_ov_word_size_run_4,
_ov_word_size_live_1,_ov_word_size_live_2,_ov_word_size_live_3,_ov_word_size_live_4,
PHYS_WORD_SIZE(_ov_word_size_run_1,_ov_word_size_live_1),
PHYS_WORD_SIZE(_ov_word_size_run_2,_ov_word_size_live_2),
PHYS_WORD_SIZE(_ov_word_size_run_3,_ov_word_size_live_3),
PHYS_WORD_SIZE(_ov_word_size_run_4,_ov_word_size_live_4),
};


/*  software stack to temporarily store registers corrupted by overlay manager */
.VAR  ov_stack[20];
.VAR  start_addr_stack [2] = 0,0;
.VAR  end_addr_stack [2] = 0,0;


/************************************************************************/
/*          Overlay Manager Function                        */

.SECTION/PM             pm_code;

_OverlayManager:
.GLOBAL _OverlayManager;

/*  _overlayID has been defined as R0.  R0 is set in the PLIT of LDF.   */
/*  Set up DMA transfer to internal memory through the external port.   */


/*  Store values of registers used by the overlay manager in to the */
/*  software stack.                         */
dm(ov_stack)=i7;
dm(ov_stack+1)=m7;
dm(ov_stack+2)=l7;
i7=ov_stack+3;
m7=1;
l7=0;

dm(i7,m7)=i8;
dm(i7,m7)=m8;
dm(i7,m7)=l8;
dm(i7,m7)=r2;
dm(i7,m7)=r3;
dm(i7,m7)=r4;
dm(i7,m7)=r5;
dm(i7,m7)=r6;
dm(i7,m7)=ustat1;

/* Use the overlay id as an index (must subtract one)  */
R0=R0-1; /* Overlay ID -1 */
m8=R0;   /*  Offset into the arrays containing linker */
         /*  defined overlay constants.               */

r0=i6;  dm(i7,m7)=r0;
r0=i0;  dm(i7,m7)=r0;
r0=m0;  dm(i7,m7)=r0;
r0=l0;  dm(0,i7)=r0;

l8=0; // Clear L0 & L8 to keep DAGs from using Circ Buffers
l0=0;
r2=3; // Save multiplier to convert 48-bit address to 32-bit
r6=1;

m0=m8; /* Overlay ID - 1 */

/*  Get overlay run and live addresses from memory and use to */
/*  set up the master mode DMA.                   */
i8 = _OverlayConstants->runAddresses;
i0 = _OverlayConstants->liveAddresses;

r0=0;           dm(PPCTL) = r0; dm(start_addr_stack)=r0; dm(end_addr_stack)=r0;

r0=dm(m0,i0);   dm(EIPP)=r0;

//Convert 48-bit run addr to 32-bit addr (48 * 3/2 = 32)
r4=pm(m8,i8);   r3=fext r4 by 0:16; r0=r4-r3;   r3=r3*r2 (ssi); r3=lshift r3 by -1;   r0=r0 or fdep r3 by 0:16;
dm(IIPP)=r0;
//If 48-bit address is odd, save first 16-bits to restore after the overlay is loaded.
btst r4 by 0; if sz jump addr_end;
r0=lshift r0 by 1;  dm(start_addr_stack)=r0; // Store short word address
i6=r0; r0=dm(0,i6); dm(start_addr_stack+1)=r0; // Store the affected word
r6=lshift r6 by 1;

addr_end:
i0=_OverlayConstants->runNumWords;  /*  Number of words stored in internal memory  */
                 /*  Most likely the word size will be 48 bits  */
                 /*  for instructions.              */

i8=_OverlayConstants->liveNumWords; /*  Number of words stored in external memory  */

r0=pm(m8,i8);
i8=_OverlayConstants->physicalLen;
ustat1=pm(m8,i8);
//Make sure that there is a whole number of 32-bit words in ECPP (and round up if there is not)
r0=r0+1; r0=lshift r0 by -1; bit tst ustat1 16; if tf jump (pc,4);
r0=r0+1; r0=lshift r0 by -1; r0=lshift r0 by 1;
r0=lshift r0 by 1;   dm(ECPP)=r0;
r0=dm(m0,i0);
//If the overlay section will end on an even address, save the last 16-bits to restore
r4=r4+r0; btst r4 by 0; if sz jump save_internal_count;
r3=fext r4 by 0:16; r4=r4-r3;   r3=r3*r2 (ssi); r3=lshift r3 by -1;   r4=r4 or fdep r3 by 0:16;
r4=lshift r4 by 1;  r4=r4+1; dm(end_addr_stack)=r4;
i6=r4; r4=dm(0,i6); dm(end_addr_stack+1)=r4;
save_internal_count:
r0=r0*r2 (ssi); r0=r0+1; r0=lshift r0 by -1;   dm(ICPP)=r0;


r0=1;           dm(EMPP)=r0;
dm(IMPP)=r0;

//Determine if the external memory is 8- or 16-bit width
r6=lshift r6 by -1;
r0=dm(ICPP);
r3=dm(ECPP);
r0=lshift r0 by 1;
comp(r0,r3);
if lt jump external8;

//Set up for 16-bit external memory
external16:
r0=dm(ICPP);
r0=r0+r6;
dm(ICPP)=r0;
r0=dm(EIPP);
r0=r0-r6;
dm(EIPP)=r0;
r6=lshift r6 by 1;
r3=r3+r6;
dm(ECPP)=r3;
ustat1=PPBHC|PPDUR4|PP16;
dm(PPCTL)=ustat1;
jump startdma;

//Set up for 8-bit external memory
external8:
r0=dm(ICPP);
r0=r0+r6;
dm(ICPP)=r0;
r6=lshift r6 by 1;
r0=dm(EIPP);
r0=r0-r6;
dm(EIPP)=r0;
r6=lshift r6 by 1;
r3=r3+r6;
dm(ECPP)=r3;
ustat1=PPBHC|PPDUR4;
dm(PPCTL)=ustat1;

startdma:
bit set ustat1 PPEN|PPDEN;
dm(PPCTL)=ustat1;
nop;nop;nop;
/*  Wait for DMA to complete.  Note that, in this example's
    code, the DMA may not complete if another interrupt fires
    before the DMA's completion.  If this is consideration
    in your system, be sure to add this check to this code.  */
dma1_wait:      idle;

    //-------- Wait for PPI DMA to complete using polling------
    r0=dm(PPCTL);
    btst r0 by 17;
    if not sz jump (pc,-2);

/*  Restore register values from stack  */
m7=-1;
r0=dm(i7,m7);   l0=r0;
r0=dm(i7,m7);   m0=r0;
r0=dm(i7,m7);   i0=r0;
r0=dm(i7,m7);   i6=r0;
ustat1=dm(i7,m7);
r6=dm(i7,m7);
r5=dm(i7,m7);
r4=dm(i7,m7);
r3=dm(i7,m7);
r2=dm(i7,m7);
l8=dm(i7,m7);
i8=r1;
m8=0;

/*  Flush the cache.  If an instruction in previous overlay  */
/*  had been cached, it may be executed instead of the       */
/*  current overlays instruction.  (If pm transfers align.)  */
flush cache;

//Restore the saved 16-bit words corrupted by 32-bit to 48-bit conversion
r0=dm(start_addr_stack);
r1=0;
comp(r1,r0);
if eq jump check_end_stack;
r1=i6;
i6=r0;
r0=dm(start_addr_stack+1);
dm(0,i6)=r0;
i6=r1;

check_end_stack:
r0=dm(end_addr_stack);
r1=0;
comp(r1,r0);
if eq jump overlay_manager_end;
r1=i6;
i6=r0;
r0=dm(end_addr_stack+1);
dm(0,i6)=r0;
i6=r1;

overlay_manager_end:
r1=dm(i7,m7);
r0=dm(0,i7);

i7=dm(ov_stack);
m7=dm(ov_stack+1);
l7=dm(ov_stack+2);
nop;
/*  Jump to the location of the function to be executed.  */
jump (m8,i8) (db);
i8=r0;
m8=r1;

_OverlayManager.end:


/*                                              */
/************************************************************************/


