/******************************************************************************************/
/*                                                                                        */
/* File Name:  Ovl_Sec_Info.asm                                                           */
/*                                                                                        */
/* Date:  August 27, 2003                                                                 */
/*                                                                                        */
/* Purpose:  This file parses all the info collected in the Ovl_Init.asm file             */
/*           for each overlay.  It checks for the number of sections within each          */
/*           overlay.  It also checks each overlay section's type and size.  It           */
/*           accounts for the overlay id that's included in the data for each             */
/*           overlay by adding 6 words to the live address.  It accounts                  */
/*           for the overlay id by subtracting 1 from the count size of the overlay.      */
/*           Then, the information is written to the respective address and size buffers. */
/*                                                                                        */
/******************************************************************************************/

#include "ovlay.h"

#include <def21262.h>
.extern runwordSize32;
.extern runAddresses;
.extern runWordSize;

.extern num_ovl_sec;
.extern read_base_addr;
.extern read_buffer_addr;
.extern read_buffer_length;
.extern read_data_bytes;
.extern total_live_addr;
.extern total_sec_size;
.extern total_sec_type;

.section/dm dm_data;
.var longest = 0;

.segment/pm pm_code;
.global _OvlSecInfo;
.global _OvlSecInfo.end;
.global longest;

_OvlSecInfo:


/* This first section of code checks the number of sections there are within each overlay */

                lcntr = NUMBER_OF_OVERLAYS;             //count of overlays in this project

                I12 = runWordSize32;
                I14 = runWordSize;
                I15 = runAddresses;
                I10 = total_sec_size;
                I13 = num_ovl_sec;
                I9 = total_live_addr;

                //R5=1;
                do check_ovl.end until lce;
    check_ovl:  R12=0;                  //R12 stores number of sections.  Initiate to 0 to start.

                R9 = PM(I12,1);     //read the total overlay size in 32-bit words
repeat_check:   R8 = PM(I10,1);     //read the individual section size

                R12 = R12+1;            //increment the total number of sections by 1

                R9 = R9-R8;                 //subtract section size from total overlay size
                if gt jump repeat_check;    //if not equal, then there are more sections in this
                                            //overlay.  repeat the check.
                nop;nop;
                
                R9 = PM(I14,1);
                R8 = DM(longest);
                comp(R8,R9);
                if gt jump odd_test;
                DM(longest) = R9;
                
odd_test:       btst R9 by 0;             // test for odd count of 48-bit words
                if sz jump check_ovl.end;
                
                R9 = PM(I15,1);
                btst R9 by 0;             // test for odd Run address
                if sz jump check_ovl.end;
                
                R9 = PM(-1,I10);          //If both count and the start address are odd reduce the section size by 1

                R9 = R9 - 1;
                PM(-1,I10) = R9;


check_ovl.end:  PM(I13,1)=R12;      //if equal, then there are no more sections in this overlay
                                        //write number of sections in each overlay to buffer

                rts;

    _OvlSecInfo.end:
