/*  The OVLY_MGR.ASM file is the overlay manager.  When a symbol  */
/*  residing in overlay is referenced, the ovelay manager loads   */
/*  the overlay code and begins execution.  A DMA transfer is  */
/*  performed to load in the meory overlay.           */

#include <def21364.h>


/*  The following constants are defined by the linker.  */
/*  These constants contain the word size, live locaton */
/*  and run location of the overlay functions.          */
#define PHYS_WORD_SIZE(live_size) 3*live_size/2


.EXTERN read_base_addr;
.EXTERN read_buffer_addr;
.EXTERN read_buffer_length;
.EXTERN read_data_bytes;

.EXTERN _ov_word_size_run_1;
.EXTERN _ov_word_size_run_2;
.EXTERN _ov_word_size_run_3;
.EXTERN _ov_word_size_run_4;
.EXTERN _ov_runtimestartaddress_1;
.EXTERN _ov_runtimestartaddress_2;
.EXTERN _ov_runtimestartaddress_3;
.EXTERN _ov_runtimestartaddress_4;




.SECTION/DM             dm_data;


/*  Placing the linker constants in a structure so the overlay  */
/*  manager can use the appropriate constant based on the    */
/*  overlay id.                          */
.var runWordSize[4] = _ov_word_size_run_1,
                      _ov_word_size_run_2,
                      _ov_word_size_run_3,
                      _ov_word_size_run_4;
.global runWordSize;

.var runWordSize32[4] = PHYS_WORD_SIZE(_ov_word_size_run_1),
                      PHYS_WORD_SIZE(_ov_word_size_run_2),
                      PHYS_WORD_SIZE(_ov_word_size_run_3),
                      PHYS_WORD_SIZE(_ov_word_size_run_4);                      
.global runWordSize32;


.var runAddresses[4] = _ov_runtimestartaddress_1,
                       _ov_runtimestartaddress_2,
                       _ov_runtimestartaddress_3,
                       _ov_runtimestartaddress_4;
.global runAddresses;

.var num_ovl_sec[4];
.global num_ovl_sec;

.var total_live_addr[10];
.global total_live_addr;

.var total_sec_size[10];
.global total_sec_size;

.var total_sec_type[10];
.global total_sec_type;

.var read_xtra_word;

.var count_sec_size;
.global count_sec_size;


.VAR current_overlay = 0;

/*  software stack to temporarily store registers corrupted by overlay manager */
.VAR  ov_stack[20];
.VAR  start_addr_stack [2] = 0,0;
.VAR  end_addr_stack [2] = 0,0;


/************************************************************************/
/*          Overlay Manager Function                        */

.SECTION/PM             pm_code;

_OverlayManager:
.GLOBAL _OverlayManager;

/*  _overlayID has been defined as R0.  R0 is set in the PLIT of LDF.   */
/*  Set up DMA transfer to internal memory through the external port.   */

/*  Store values of registers used by the overlay manager in to the */
/*  software stack.                         */
dm(ov_stack)=i7;
dm(ov_stack+1)=m7;
dm(ov_stack+2)=l7;

/* Check to see if overlay to execute is already loaded. */
l7=r1;

i7=0; dm(start_addr_stack)=r0; dm(end_addr_stack)=r0;
i7=r0;

r1=dm(current_overlay);
dm(current_overlay)=r0;
r0=r0-r1;
r1=l7;
r0=pass r0;
if ne jump load_new_overlay;
jump overlay_loaded;

//If overlay is not loaded, load it
load_new_overlay:
r0=i7;
i7=ov_stack+3;
m7=1;
l7=0;

//Continue to store registers in the software stack
dm(i7,m7)=i8;
dm(i7,m7)=m8;
dm(i7,m7)=l8;
dm(i7,m7)=i9;
dm(i7,m7)=i10;
dm(i7,m7)=r2;
dm(i7,m7)=r3;
dm(i7,m7)=r4;
dm(i7,m7)=r5;
dm(i7,m7)=r6;
dm(i7,m7)=ustat1;

/* Use the overlay id as an index (must subtract one)  */
R0=R0-1; /* Overlay ID -1 */
m8=R0;   /*  Offset into the arrays containing linker */
         /*  defined overlay constants.               */
R5=R1;

r0=i6;  dm(i7,m7)=r0;
r0=i0;  dm(i7,m7)=r0;
r0=m0;  dm(i7,m7)=r0;
r0=l0;  dm(0,i7)=r0;

l8=0; // Clear L0 & L8 to keep DAGs from using Circ Buffers
l0=0;
r2=3; // Save multiplier to convert 48-bit address to 32-bit
r6=0;

m0=m8; /* Overlay ID - 1 */

/*  Get overlay run and live addresses from memory and use to */
/*  set up the master mode DMA.                   */
i8 = runAddresses;
r0=0;           dm(SPICTL) = r0;

//Convert 48-bit run addr to 32-bit addr (48 * 3/2 = 32)
r4=pm(m8,i8);   r3=fext r4 by 0:17; r0=r4-r3;   r3=r3*r2 (ssi); r3=lshift r3 by -1;   r0=r0+r3;
dm(read_buffer_addr)=r0;
//If 48-bit address is odd, save first 16-bits to restore after the overlay is loaded. Use r6 to indicate this.
btst r4 by 0; if sz jump addr_end;
r6=1;

addr_end:
i0=runWordSize;  /*  Fetch the number of words to be stored in internal memory  */
r0=dm(m0,i0);

/* If the block will end on an even 48-bit address, save the last 16-bits to restore */
r4=r4+r0; btst r4 by 0; if sz jump get_ovl_start;
/* Convert the address to its 16-bit equivalent */
r3=fext r4 by 0:17; r4=r4-r3;   r3=r3*r2 (ssi); r3=lshift r3 by -1;   r4=r4+ r3;
r4=lshift r4 by 1;  r4=r4+1; dm(end_addr_stack)=r4;
i6=r4; r4=dm(0,i6); dm(end_addr_stack+1)=r4;

/* Start fetching the overlay */
get_ovl_start:
/* Determine which overlay is to be loaded. */
i0=num_ovl_sec;
r4=0;
r3=m0;
r3=pass r3;
if eq jump start_ovl_fetch;
/*If it is not the first overlay, determine how many sections preceed the overlay to be loaded. Store this in r4*/
lcntr = r3;
do (pc,2) until lce;
r0=dm(i0,1);
r4=r4+r0;

/* Set up the DAG registers to fetch the overlay */
start_ovl_fetch:
i0=num_ovl_sec;
m8=r4;
i8=total_live_addr;
modify(i8,m8);
i9=total_sec_size;
modify(i9,m8);
i10=total_sec_type;
modify(i10,m8);

/* Fetch the number of sections as indicated in num_ovl_sec */
lcntr = dm(m0,i0);
do fetch_overlay_end until lce;

/* If r6 is 1, save the first 16 bits */
btst r6 by 0;
if sz jump ready_to_fetch;
r0=dm(read_buffer_addr);
r0=lshift r0 by 1;  dm(start_addr_stack)=r0; // Store short word address
i6=r0; r0=dm(0,i6); dm(start_addr_stack+1)=r0; // Store the affected word

/* Determine the type of section to be fetched, either init or zero */
ready_to_fetch:
r0=pm(i10,1);
r2=9;
comp(r0,r2);
if lt jump init_zeros;

/* If it is an init section, transfer the data from the flash */
init_data:
/* Must save the 32-bit word that preceeds the section to initialize */
/* Always transfer 1 extra word, corresponding to the SPI read command */
/* If r6 is 1, change the start address and count for the extra word to transfer */
/* Set read_buffer_addr, read_buffer_length, and read_base_addr before calling read_data_bytes,
these buffers hold the internal memory address, 32-bit word count, and address in the flash, respectively */
r0=dm(read_buffer_addr);
r0=r0-1;
dm(read_buffer_addr)=r0;
i0=r0;
r0=dm(i0,0);
dm(read_xtra_word)=r0;
r0=pm(i9,0);
r0=r0+r6;
r0=r0+1;
dm(read_buffer_length)=r0;
r6=lshift r6 by 1;
r1=pm(i8,1);
r1=r1-r6;
dm(read_base_addr)=r1;
r6=lshift r6 by -1;
call read_data_bytes;
/* Restore the extra word overwritten during the read command */
r0=dm(read_xtra_word);
dm(i0,0)=r0;
r0=dm(read_buffer_addr);
r0=r0+1;
dm(read_buffer_addr)=r0;
jump update_for_next_pass;

/* If its a zero section, write the zeros directly */
init_zeros:
/* If r6 is 1, change the count for the extra word to transfer */
modify(i8,1);
r0=pm(i9,0);
r0=r0+r6;
r2=0;
r1=dm(read_buffer_addr);
i6=r1;
lcntr=r0, do (pc,1) until lce;
dm(i6,1)=r2;

update_for_next_pass:
/* Restore the first 16-bits if it will be affected */
btst r6 by 0;
if sz jump update_buffer_addr;
i6=dm(start_addr_stack); // Get short word address
r0=dm(start_addr_stack+1); // Get the affected word
dm(0,i6)=r0;

/* Update the internal buffer address to point to the beginning of the next section */
update_buffer_addr:
    r0=pm(i9,1);
    r1=DM(read_buffer_addr);
    r1=r1+r0;
 
fetch_overlay_end:
    dm(read_buffer_addr)=r1;

/*  Restore register values from stack  */
m7=-1;
r1=r5;
restore_dag1:
r0=dm(i7,m7);   l0=r0;
r0=dm(i7,m7);   m0=r0;
r0=dm(i7,m7);   i0=r0;
r0=dm(i7,m7);   i6=r0;
restore_nondag1:
ustat1=dm(i7,m7);
r6=dm(i7,m7);
r5=dm(i7,m7);
r4=dm(i7,m7);
r3=dm(i7,m7);
r2=dm(i7,m7);
i10=dm(i7,m7);
i9=dm(i7,m7);
l8=dm(i7,m7);

overlay_loaded:
i8=r1;
m8=0;

/*  Flush the cache.  If an instruction in previous overlay  */
/*  had been cached, it may be executed instead of the       */
/*  current overlays instruction.  (If pm transfers align.)  */
flush cache;

check_end_stack:
r0=dm(end_addr_stack);
r1=0;
comp(r1,r0);
if eq jump overlay_manager_end;
r1=i6;
i6=r0;
r0=dm(end_addr_stack+1);
dm(0,i6)=r0;
i6=r1;

overlay_manager_end:
r1=dm(i7,m7);
r0=dm(0,i7);

i7=dm(ov_stack);
m7=dm(ov_stack+1);
l7=dm(ov_stack+2);
nop;
/*  Jump to the location of the function to be executed.  */
jump (m8,i8) (db);
i8=r0;
m8=r1;

_OverlayManager.end:


/*                                              */
/************************************************************************/


